@@ -2,7 +2,9 @@ import dask  # noqa
 import numpy as np
 import xarray as xr
 import xclim as xc
-from xclim.core.calendar import percentile_doy
+from xclim.core.calendar import percentile_doy, resample_doy
+from xclim.core.units import convert_units_to, rate2amount
+from xclim.indices.generic import compare
 
 
 def _thresh_string(thresh, units):
@@ -21,6 +23,10 @@ def _get_da(dictionary, var):
 
 
 def _get_percentile(da, perc, base_period_time_range):
+    if isinstance(perc, xr.Dataset):
+        return perc["per"]
+    elif isinstance(perc, xr.DataArray):
+        return perc
     tslice = slice(base_period_time_range[0], base_period_time_range[1])
     base_period = da.sel(time=tslice)
     with dask.config.set(**{"array.slicing.split_large_chunks": False}):
@@ -40,8 +46,15 @@ class CD:
     """Number of cold and dry days (tas, pr)."""
 
     base_period_time_range = BASE_PERIOD
+    perc_tas = None
+    perc_pr = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc_tas=perc_tas,
+        perc_pr=perc_pr,
+        **params,
+    ):
         """Calculate number of cold and dry days.
 
         Parameters
@@ -55,20 +68,22 @@ class CD:
         """
         da_tas = _get_da(params, "tas")
         da_pr = _get_da(params, "pr")
-        percentile_tas = _get_percentile(
-            da=da_tas,
-            perc=25,
-            base_period_time_range=base_period_time_range,
-        )
-        percentile_pr = _get_percentile(
-            da=da_pr,
-            perc=25,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc_tas is None:
+            perc_tas = _get_percentile(
+                da=da_tas,
+                perc=25,
+                base_period_time_range=base_period_time_range,
+            )
+        if perc_pr is None:
+            perc_pr = _get_percentile(
+                da=da_pr,
+                perc=25,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.cold_and_dry_days(
-                tas_per=percentile_tas,
-                pr_per=percentile_pr,
+                tas_per=perc_tas,
+                pr_per=perc_pr,
                 **params,
             )
 
@@ -139,8 +154,13 @@ class CSDI:
     """Cold spell duration index (tasmin)."""
 
     base_period_time_range = BASE_PERIOD
+    perc = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc=perc,
+        **params,
+    ):
         """Calculate cold spell duration index.
 
         Parameters
@@ -154,14 +174,15 @@ class CSDI:
         At least 6 consecutive days.
         """
         da = _get_da(params, "tasmin")
-        percentile = _get_percentile(
-            da=da,
-            perc=10,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc is None:
+            perc = _get_percentile(
+                da=da,
+                perc=10,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.cold_spell_duration_index(
-                tasmin_per=percentile,
+                tasmin_per=perc,
                 window=6,
                 **params,
             )
@@ -195,8 +216,15 @@ class CW:
     """Number of cold and wet days (tas, pr)."""
 
     base_period_time_range = BASE_PERIOD
+    perc_tas = None
+    perc_pr = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc_tas=perc_tas,
+        perc_pr=perc_pr,
+        **params,
+    ):
         """Calculate number of cold and wet days.
 
         Parameters
@@ -210,20 +238,22 @@ class CW:
         """
         da_tas = _get_da(params, "tas")
         da_pr = _get_da(params, "pr")
-        percentile_tas = _get_percentile(
-            da=da_tas,
-            perc=25,
-            base_period_time_range=base_period_time_range,
-        )
-        percentile_pr = _get_percentile(
-            da=da_pr,
-            perc=75,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc_tas is None:
+            perc_tas = _get_percentile(
+                da=da_tas,
+                perc=25,
+                base_period_time_range=base_period_time_range,
+            )
+        if perc_pr is None:
+            perc_pr = _get_percentile(
+                da=da_pr,
+                perc=75,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.cold_and_wet_days(
-                tas_per=percentile_tas,
-                pr_per=percentile_pr,
+                tas_per=perc_tas,
+                pr_per=perc_pr,
                 **params,
             )
 
@@ -466,6 +496,29 @@ class RR:
         return xc.atmos.precip_accumulation(**params)
 
 
+class RRm:
+    """Mean daily precipitation (pr)."""
+
+    def compute(**params):
+        """Calculate mean daily precipitation.
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+        xarray.DataArray
+            Mean daily precipitation.
+        """
+        da = _get_da(params, "pr")
+        pram = rate2amount(da)
+        return (
+            pram.resample(time=params["freq"])
+            .mean(dim="time")
+            .assign_attrs(units=pram.units)
+        )
+
+
 class RR1:
     """Number of wet days (pr)."""
 
@@ -554,7 +607,36 @@ class R25mm:
         )
 
 
-class RDYYp:
+class RRYYp:
+    """Precip percentil value (pr)."""
+
+    perc = 75
+    base_period_time_range = BASE_PERIOD
+
+    def compute(
+        perc=perc,
+        base_period_time_range=base_period_time_range,
+        **params,
+    ):
+        """Calculate precip percentile value.
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+        xarray.DataArray
+            Precip percentil value.
+        """
+        da = _get_da(params, "pr")
+        return _get_percentile(
+            da=da,
+            perc=perc,
+            base_period_time_range=base_period_time_range,
+        )
+
+
+class RYYp:
     """Number of wet days with precip over a given percentile (pr)."""
 
     perc = 75
@@ -658,6 +740,57 @@ class RXYYday:
         )
 
 
+class RYYpABS:
+    """Total precipitation amount with precip > {perc}th percentile (pr)."""
+
+    perc = 75
+    base_period_time_range = BASE_PERIOD
+
+    def compute(
+        perc=perc,
+        base_period_time_range=base_period_time_range,
+        **params,
+    ):
+        """Calculate total precipitation amount
+        with precip > {perc}th percentile.
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+        xarray.DataArray
+            Precipitation fraction with precip > {perc}th percentile.
+        """
+        da = _get_da(params, "pr")
+        pr_per = _get_percentile(
+            da=da,
+            perc=perc,
+            base_period_time_range=base_period_time_range,
+        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            pr_per = convert_units_to(pr_per, da, context="hydro")
+            thresh = convert_units_to("1 mm/day", da, context="hydro")
+
+            tp = pr_per.where(pr_per > thresh, thresh)
+            if "dayofyear" in pr_per.coords:
+                # Create time series out of doy values.
+                tp = resample_doy(tp, da)
+
+            constrain = (">", ">=")
+
+            # Compute the days when precip is both over the wet day threshold
+            # and the percentile threshold.
+            over = (
+                da.where(compare(da, ">", tp, constrain))
+                .resample(time=params["freq"])
+                .sum(dim="time")
+            )
+            out = convert_units_to(over, "mm/day")
+            out.attrs["units"] = "mm"
+            return out
+
+
 class RYYpTOT:
     """Precipitation fraction with precip > {perc}th percentile (pr)."""
 
@@ -786,8 +919,13 @@ class TG10p:
     """Fraction of days with mean temperature < 10th percentile (tas)."""
 
     base_period_time_range = BASE_PERIOD
+    perc = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc=perc,
+        **params,
+    ):
         """Calculate fraction of days with mean temperature < 10th percentile.
 
         Parameters
@@ -801,14 +939,15 @@ class TG10p:
             Fraction of days with mean temperature < 10th percentile".
         """
         da = _get_da(params, "tas")
-        percentile = _get_percentile(
-            da=da,
-            perc=10,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc is None:
+            perc = _get_percentile(
+                da=da,
+                perc=10,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.tg10p(
-                tas_per=percentile,
+                tas_per=perc,
                 **params,
             )
 
@@ -817,8 +956,13 @@ class TG90p:
     """Fraction of days with mean temperature > 90th percentile (tas)."""
 
     base_period_time_range = BASE_PERIOD
+    perc = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc=perc,
+        **params,
+    ):
         """Calculate fraction of days with mean temperature > 90th percentile".
 
         Parameters
@@ -832,14 +976,15 @@ class TG90p:
             Fraction of days with mean temperature > 90th percentile".
         """
         da = _get_da(params, "tas")
-        percentile = _get_percentile(
-            da=da,
-            perc=90,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc is None:
+            perc = _get_percentile(
+                da=da,
+                perc=90,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.tg90p(
-                tas_per=percentile,
+                tas_per=perc,
                 **params,
             )
 
@@ -892,8 +1037,13 @@ class TX10p:
     """Fraction of days with max temperature < 10th percentile (tasmax)."""
 
     base_period_time_range = BASE_PERIOD
+    perc = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc=perc,
+        **params,
+    ):
         """Calculate fraction of days with max temperature < 10th percentile.
 
         Parameters
@@ -907,14 +1057,15 @@ class TX10p:
             Fraction of days with maximum temperature < 10th percentile".
         """
         da = _get_da(params, "tasmax")
-        percentile = _get_percentile(
-            da=da,
-            perc=10,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc is None:
+            perc = _get_percentile(
+                da=da,
+                perc=10,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.tx10p(
-                tasmax_per=percentile,
+                tasmax_per=perc,
                 **params,
             )
 
@@ -923,8 +1074,13 @@ class TX90p:
     """Fraction of days with max temperature > 90th percentile (tasmax)."""
 
     base_period_time_range = BASE_PERIOD
+    perc = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc=perc,
+        **params,
+    ):
         """Calculate fraction of days with max temperature > 90th percentile.
 
         Parameters
@@ -938,14 +1094,15 @@ class TX90p:
             Fraction of days with maximum temperature > 90th percentile".
         """
         da = _get_da(params, "tasmax")
-        percentile = _get_percentile(
-            da=da,
-            perc=90,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc is None:
+            perc = _get_percentile(
+                da=da,
+                perc=90,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.tx90p(
-                tasmax_per=percentile,
+                tasmax_per=perc,
                 **params,
             )
 
@@ -1011,8 +1168,13 @@ class TN10p:
     """Fraction of days with min temperature < 10th percentile."""
 
     base_period_time_range = BASE_PERIOD
+    perc = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc=perc,
+        **params,
+    ):
         """Calculate fraction of days with min temperature < 10th percentile.
 
         Parameters
@@ -1026,14 +1188,15 @@ class TN10p:
             Fraction of days with minimum temperature < 10th percentile".
         """
         da = _get_da(params, "tasmin")
-        percentile = _get_percentile(
-            da=da,
-            perc=10,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc is None:
+            perc = _get_percentile(
+                da=da,
+                perc=10,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.tn10p(
-                tasmin_per=percentile,
+                tasmin_per=perc,
                 **params,
             )
 
@@ -1042,8 +1205,13 @@ class TN90p:
     """Fraction of days with min temperature > 90th percentile."""
 
     base_period_time_range = BASE_PERIOD
+    perc = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc=perc,
+        **params,
+    ):
         """Calculate fraction of days with min temperature > 90th percentile.
 
         Parameters
@@ -1057,14 +1225,15 @@ class TN90p:
             Fraction of days with minimum temperature > 90th percentile".
         """
         da = _get_da(params, "tasmin")
-        percentile = _get_percentile(
-            da=da,
-            perc=90,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc is None:
+            perc = _get_percentile(
+                da=da,
+                perc=90,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.tn90p(
-                tasmin_per=percentile,
+                tasmin_per=perc,
                 **params,
             )
 
@@ -1111,8 +1280,15 @@ class WD:
     """Number of warm and dry days (tas, pr)."""
 
     base_period_time_range = BASE_PERIOD
+    perc_tas = None
+    perc_pr = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc_tas=perc_tas,
+        perc_pr=perc_pr,
+        **params,
+    ):
         """Calculate number of warm and dry days.
 
         Parameters
@@ -1126,20 +1302,22 @@ class WD:
         """
         da_tas = _get_da(params, "tas")
         da_pr = _get_da(params, "pr")
-        percentile_tas = _get_percentile(
-            da=da_tas,
-            perc=75,
-            base_period_time_range=base_period_time_range,
-        )
-        percentile_pr = _get_percentile(
-            da=da_pr,
-            perc=25,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc_tas is None:
+            perc_tas = _get_percentile(
+                da=da_tas,
+                perc=75,
+                base_period_time_range=base_period_time_range,
+            )
+        if perc_pr is None:
+            perc_pr = _get_percentile(
+                da=da_pr,
+                perc=25,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.warm_and_dry_days(
-                tas_per=percentile_tas,
-                pr_per=percentile_pr,
+                tas_per=perc_tas,
+                pr_per=perc_pr,
                 **params,
             )
 
@@ -1148,8 +1326,13 @@ class WSDI:
     """Warm spell duration index (tasmax)."""
 
     base_period_time_range = BASE_PERIOD
+    perc = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc=perc,
+        **params,
+    ):
         """Calculate warm spell duration index.
 
         Parameters
@@ -1163,14 +1346,15 @@ class WSDI:
         At least 6 consecutive days.
         """
         da = _get_da(params, "tasmax")
-        percentile = _get_percentile(
-            da=da,
-            perc=90,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc is None:
+            perc = _get_percentile(
+                da=da,
+                perc=90,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.warm_spell_duration_index(
-                tasmax_per=percentile,
+                tasmax_per=perc,
                 window=6,
                 **params,
             )
@@ -1180,8 +1364,15 @@ class WW:
     """Number of warm and wet days (tas, pr)."""
 
     base_period_time_range = BASE_PERIOD
+    perc_tas = None
+    perc_pr = None
 
-    def compute(base_period_time_range=base_period_time_range, **params):
+    def compute(
+        base_period_time_range=base_period_time_range,
+        perc_tas=perc_tas,
+        perc_pr=perc_pr,
+        **params,
+    ):
         """Calculate number of warm and wet days.
 
         Parameters
@@ -1195,20 +1386,22 @@ class WW:
         """
         da_tas = _get_da(params, "tas")
         da_pr = _get_da(params, "pr")
-        percentile_tas = _get_percentile(
-            da=da_tas,
-            perc=75,
-            base_period_time_range=base_period_time_range,
-        )
-        percentile_pr = _get_percentile(
-            da=da_pr,
-            perc=75,
-            base_period_time_range=base_period_time_range,
-        )
+        if perc_tas is None:
+            perc_tas = _get_percentile(
+                da=da_tas,
+                perc=75,
+                base_period_time_range=base_period_time_range,
+            )
+        if perc_pr is None:
+            perc_pr = _get_percentile(
+                da=da_pr,
+                perc=75,
+                base_period_time_range=base_period_time_range,
+            )
         with dask.config.set(**{"array.slicing.split_large_chunks": False}):
             return xc.atmos.warm_and_wet_days(
-                tas_per=percentile_tas,
-                pr_per=percentile_pr,
+                tas_per=perc_tas,
+                pr_per=perc_pr,
                 **params,
             )
 
@@ -1314,6 +1507,7 @@ class SD:
         thresh = _thresh_string(thresh, "mm/day")
         da = _get_da(params, "prsn")
         prsn = _convert_snow_mm_day(da)
+        prsn = prsn.assign_attrs(**da.attrs)
         prsn.attrs["units"] = "kg m-2 s-1"
         if "ds" in params.keys():
             del params["ds"]
@@ -1353,6 +1547,7 @@ class Sint:
 
     def compute(**params):
         """Calculate snowfall intensity.
+
         Parameters
         ----------
 
@@ -1375,6 +1570,7 @@ class Sfreq:
 
     def compute(**params):
         """Calculate snowfall frequency.
+
         Parameters
         ----------
         For input parameters see:
@@ -1387,6 +1583,7 @@ class Sfreq:
         thresh = _thresh_string(1, "mm/day")
         da = _get_da(params, "prsn")
         prsn = _convert_snow_mm_day(da)
+        prsn = prsn.assign_attrs(**da.attrs)
         prsn.attrs["units"] = "kg m-2 s-1"
         if "ds" in params.keys():
             del params["ds"]
@@ -1396,7 +1593,8 @@ class Sfreq:
             **params,
         )
         ndays = da.resample(time=params["freq"]).count(dim="time")
-        return sd / ndays * 100
+        sfreq = sd / ndays * 100
+        return sfreq.assign_attrs(**sd.attrs)
 
 
 class UTCI:
@@ -1447,3 +1645,137 @@ class WI:
             thresh=thresh,
             **params,
         )
+
+
+class HW:
+    """Maximum length of heat waves (tasmax, tasmin)."""
+
+    thresh_tasmin = 22
+    thresh_tasmax = 30
+
+    def compute(
+        thresh_tasmax=thresh_tasmax,
+        thresh_tasmin=thresh_tasmin,
+        **params,
+    ):
+        """Calculate maximum number of heat waves.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#heat_wave_max_length
+
+        Returns
+        -------
+        xarray.DataArray
+            Maximum length of heat waves.
+        """
+        thresh_tasmax = _thresh_string(thresh_tasmax, "degC")
+        thresh_tasmin = _thresh_string(thresh_tasmin, "degC")
+        return xc.atmos.heat_wave_max_length(
+            thresh_tasmax=thresh_tasmax,
+            thresh_tasmin=thresh_tasmin,
+            **params,
+        )
+
+
+class GSS:
+    """Growing season start (tas)."""
+
+    thresh = 5
+    window = 5
+
+    def compute(thresh=thresh, window=window, **params):
+        """Calculate growing season start.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#growing_season_start
+
+        Returns
+        -------
+        xarray.DataArray
+            Growing season start.
+        """
+        thresh = _thresh_string(thresh, "degC")
+        return xc.atmos.growing_season_start(
+            thresh=thresh,
+            window=window,
+            **params,
+        )
+
+
+class GSE:
+    """Growing season end (tas)."""
+
+    thresh = 5
+    window = 5
+
+    def compute(thresh=thresh, window=window, **params):
+        """Calculate growing season end.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#growing_season_end
+
+        Returns
+        -------
+        xarray.DataArray
+            Growing season end.
+        """
+        thresh = _thresh_string(thresh, "degC")
+        return xc.atmos.growing_season_end(
+            thresh=thresh,
+            window=window,
+            **params,
+        )
+
+
+class FFS:
+    """Frost free season start (tasmin)."""
+
+    window = 5
+
+    def compute(window=window, **params):
+        """Calculate frost free season start.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#frost_free_season_start
+
+        Returns
+        -------
+        xarray.DataArray
+            Frost free season start.
+        """
+        return xc.atmos.frost_free_season_start(
+            window=window,
+            **params,
+        )
+
+
+class FFE:
+    """Frost free season end (tasmin)."""
+
+    window = 5
+
+    def compute(window=window, **params):
+        """Calculate frost free season end.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#frost_free_season_end
+
+        Returns
+        -------
+        xarray.DataArray
+            Frost free season end.
+        """
+        return xc.atmos.frost_free_season_end(
+            window=window,
+            **params,
+        )
