@@ -7,7 +7,11 @@ from index_calculator import test_netcdf
 
 def test_processing():
     tas_ds = open_xrdataset(test_netcdf["tas"])
-    preproc = xcalc.preprocessing(tas_ds, freq="week")
+    preproc = xcalc.preprocessing(
+        tas_ds,
+        freq="week",
+        crop_time_axis=False,
+    )
     proc = xcalc.processing("TG", preproc_obj=preproc)
     postproc = xcalc.postprocessing(
         project="CORDEX",
@@ -25,6 +29,7 @@ def test_index_calculator():
         ds=tas_ds,
         freq="week",
         index="TG",
+        crop_time_axis=False,
         project="CORDEX",
         institution="test institution",
         institution_id="TEST",
@@ -39,6 +44,7 @@ def test_thresh_index_calculator():
         ds=pr_ds,
         freq="week",
         index="RX3day",
+        crop_time_axis=False,
         project="CORDEX",
         institution_id="TEST",
     )
@@ -47,6 +53,7 @@ def test_thresh_index_calculator():
         freq="week",
         index="RXYYday",
         thresh=3,
+        crop_time_axis=False,
         project="CORDEX",
         institution_id="TEST",
     )
@@ -54,6 +61,7 @@ def test_thresh_index_calculator():
         ds=pr_ds,
         freq="week",
         index="RXYYday",
+        crop_time_axis=False,
         project="CORDEX",
         institution_id="TEST",
     )
